#include "flex_io.h"

#define IMXRT_FLEXIO1_S		(*(IMXRT_FLEXIO_t *)0x401AC000)
#define IMXRT_FLEXIO2_S		(*(IMXRT_FLEXIO_t *)0x401B0000)
//#define IMXRT_FLEXIO3		(*(IMXRT_REGISTER32_t *)0x42020000) only RT1062
//                                        0     1     2      3  4  5  6  7  8
static const uint8_t FLEXIO1_pins[] = {0xff, 0xff, 0xff , 0xff, 2, 3, 4, 5, 33};

static const uint8_t FLEXIO2_pins[] = {10, 12, 11 , 13, 0xff, 0xff, 0xff , 0xff,
                                       0xff, 0xff, 8 , 9, 32, 0xff, 0xff , 0xff, 7, 6
                                      };


static FlexIOHandler flexIO1(&IMXRT_FLEXIO1_S, FLEXIO1_pins, sizeof(FLEXIO1_pins));
static FlexIOHandler flexIO2(&IMXRT_FLEXIO2_S, FLEXIO2_pins, sizeof(FLEXIO2_pins));

static FlexIOHandler *flex_list[] = {&flexIO1, &flexIO2};

FlexIOHandler *FlexIOHandler::mapIOPinToFlexIOHandler(uint8_t pin, uint8_t &flex_pin)
{
  FlexIOHandler *pflex = nullptr;

  for (uint8_t iflex = 0; iflex < (sizeof(flex_list) / sizeof(flex_list[0])); iflex++) {
    pflex = flex_list[iflex];

    for (uint8_t i = 0; i < pflex->_io_pin_list_cnt; i++ ) {
      if (pflex->_io_pin_list[i] == pin) {
        flex_pin = i;
        return pflex;
      }
    }
  }
  flex_pin = 0xff;
  return nullptr;
}

// TODO: Get count of timers/shifters/buffers out of object...
// Also handle cnt > 1...
uint8_t FlexIOHandler::requestTimers(uint8_t cnt) {
  uint8_t mask = 0x1;
  for (uint8_t i = 0; i < 4; i++) {
    if (!(_used_timers & mask)) {
      _used_timers |= mask;
      return i;
    }
  }
  return 0xff;
}

uint8_t FlexIOHandler::requestShifters(uint8_t cnt) {
  uint8_t mask = 0x1;
  for (uint8_t i = 0; i < 4; i++) {
    if (!(_used_shifters & mask)) {
      _used_shifters |= mask;
      return i;
    }
  }
  return 0xff;
}
uint8_t FlexIOHandler::requestBuffers(uint8_t cnt) {
  uint8_t mask = 0x1;
  for (uint8_t i = 0; i < 4; i++) {
    if (!(_used_buffers & mask)) {
      _used_buffers |= mask;
      return i;
    }
  }
  return 0xff;
}

void FlexIOHandler::freeTimers(uint8_t n, uint8_t cnt) {
}

void FlexIOHandler::freeShifters(uint8_t n, uint8_t cnt) {

}

void FlexIOHandler::freeBuffers(uint8_t n, uint8_t cnt) {
}
