#include "flex_io.h"

FlexIOHandler *pflex;
uint8_t flex_pin;
uint8_t transmit_timer;
uint8_t transmit_shifter;

void setup() {
  pinMode(13, OUTPUT);
  while (!Serial && millis() < 4000);
  Serial.begin(115200);
  delay(500);
  IMXRT_FLEXIO_t *p = nullptr;

  Serial.printf("Offset to SHIFTBUFNIS %x\n", (uint32_t)&p->SHIFTBUFNIS[0]);
  pflex = FlexIOHandler::mapIOPinToFlexIOHandler(2, flex_pin);
  Serial.printf("pin 2 maps to: %x, port: %x pin %x\n", (uint32_t)pflex, (uint32_t)pflex->port(), flex_pin);
  transmit_timer = pflex->requestTimers();
  transmit_shifter = pflex->requestShifters();
  Serial.printf("timer index: %d shifter index: %d\n", transmit_timer, transmit_shifter);

  // lets try to configure a tranmitter like example
  Serial.println("Enable flexio clock");
  CCM_CCGR5 |= CCM_CCGR5_FLEXIO1(CCM_CCGR_ON);
  p = pflex->port();
  Serial.println("Before configure flexio");
  p->SHIFTCFG[transmit_shifter] = FLEXIO_SHIFTCFG_SSTOP(3) | FLEXIO_SHIFTCFG_SSTART(2); //0x0000_0032;
  p->SHIFTCTL[transmit_shifter] = FLEXIO_SHIFTCTL_PINCFG(3) | FLEXIO_SHIFTCTL_SMOD(2) |
                                  FLEXIO_SHIFTCTL_TIMSEL(transmit_timer) | FLEXIO_SHIFTCTL_PINSEL(flex_pin); // 0x0003_0002;
  p->TIMCMP[transmit_timer] = 0xF01; //0x0000_0F01;		//
  p->TIMCFG[transmit_timer] = FLEXIO_TIMCFG_TSTART | FLEXIO_TIMCFG_TSTOP(2) |
                              FLEXIO_TIMCFG_TIMENA(2) |  FLEXIO_TIMCFG_TIMDIS(2); //0x0000_2222;
  p->TIMCTL[transmit_timer] = FLEXIO_TIMCTL_TIMOD(1) | FLEXIO_TIMCTL_TRGPOL | FLEXIO_TIMCTL_TRGSRC
                              | FLEXIO_TIMCTL_TRGSEL(1) | FLEXIO_TIMCTL_PINSEL(flex_pin);  // 0x01C0_0001;
  p->CTRL = FLEXIO_CTRL_FLEXEN;

  // Set the IO pin into FLEXIO mode
  *(portConfigRegister(2)) = 0x14;

  // Lets print out some of the settings and the like to get idea of state
  Serial.printf("VERID:%x PARAM:%x CTRL:%x PIN: %x\n", p->PARAM, p->CTRL, p->CTRL, p->PIN);
  Serial.printf("SHIFTSTAT:%x SHIFTERR=%x TIMSTAT=%x\n", p->SHIFTSTAT, p->SHIFTERR, p->TIMSTAT);
  Serial.printf("SHIFTSIEN:%x SHIFTEIEN=%x TIMIEN=%x\n", p->SHIFTSIEN, p->SHIFTEIEN, p->TIMIEN);
  Serial.printf("SHIFTSDEN:%x SHIFTSTATE=%x\n", p->SHIFTSDEN, p->SHIFTSTATE);
  Serial.printf("SHIFTCTL:%x %x %x %x\n", p->SHIFTCTL[0], p->SHIFTCTL[1], p->SHIFTCTL[2], p->SHIFTCTL[3]);
  Serial.printf("SHIFTCFG:%x %x %x %x\n", p->SHIFTCFG[0], p->SHIFTCFG[1], p->SHIFTCFG[2], p->SHIFTCFG[3]);
  Serial.printf("TIMCTL:%x %x %x %x\n", p->TIMCTL[0], p->TIMCTL[1], p->TIMCTL[2], p->TIMCTL[3]);
  Serial.printf("TIMCFG:%x %x %x %x\n", p->TIMCFG[0], p->TIMCFG[1], p->TIMCFG[2], p->TIMCFG[3]);
  Serial.printf("TIMCMP:%x %x %x %x\n", p->TIMCMP[0], p->TIMCMP[1], p->TIMCMP[2], p->TIMCMP[3]);

  Serial.println("End Setup");
}

uint8_t loop_char = 'a';
void loop() {
  digitalWrite(13, !digitalRead(13));
  pflex->port()->SHIFTBUF[transmit_shifter] = loop_char;
  loop_char++;
  if (loop_char > 'z') loop_char = 'a';
  delay(100);

}
